@@ -3,9 +3,9 @@ Class Definitions
 =================
 
 To define a named glyph class in the FEE language, use the ``DefineClass``
-verb. This takes three arguments: the first is a class name, which must
-start with the ``@`` character; the second is the symbol ``=``; the third
-is a glyph selector as described above::
+verb. This takes three arguments: the first is a class name, which must start
+with the ``@`` character; the second is the symbol ``=``; the third is a glyph
+selector as described above::
 
     DefineClass @upper_alts = @upper.alt;
     DefineClass @lower = /^[a-z]$/;
@@ -22,10 +22,19 @@ As well as subtracted (``-``):
     DefineClass @ABCD = A | B | C | D;
     DefineClass @ABC = @ABCD - D;
 
+Glyph classes can also be defined using the Unicode codepoints that the glyphs
+map to in the font. For example:
+
+    DefineClass @Q = U+51;
+
+In the PostScript standard, the ASCII digits don't have regex friendly names,
+so you may wish to select them with a Unicode range selector:
+
+    DefineClass @digits = U+30=>U+39;
+
 Finally, glyph classes can be filtered through the use of one or more
-*predicates*, which take the form ``and`` followed by a
-bracketed relationship, and which tests the properties of the glyphs
-against the expression given::
+*predicates*, which take the form ``and`` followed by a bracketed relationship,
+and which tests the properties of the glyphs against the expression given::
 
     DefineClass @short_behs = /^BE/ and (width < 200);
 
@@ -41,15 +50,15 @@ against the expression given::
   - ``rise`` (difference in Y coordinate between cursive entry and exit)
   - ``fullwidth`` (``xMax``-``xMin``)
 
-- The second part is a comparison operator (``>=``, ``<=``,
-  ``=``, ``<``, or ``>``).
+- The second part is a comparison operator (``>=``, ``<=``, ``=``, ``<``, or
+  ``>``).
 
-- The third is either an integer or a metric name and the name of a
-  single glyph in brackets.
+- The third is either an integer or a metric name and the name of a single
+  glyph in brackets.
 
-This last form is best understood by example. The following definition
-selects all members of the glyph class ``@alpha`` whose advance width is
-less than the advance width of the ``space`` glyph::
+This last form is best understood by example. The following definition selects
+all members of the glyph class ``@alpha`` whose advance width is less than the
+advance width of the ``space`` glyph::
 
     DefineClass @shorter_than_space = @alpha and (width < width(space));
 
@@ -101,22 +110,24 @@ will take any glyph selector and display its contents on standard error.
 
 import re
 from glyphtools import get_glyph_metrics, bin_glyphs_by_metric
+from bidict import ValueDuplicationError
 
 import warnings
 
 
 GRAMMAR = """
-predicate = ws 'and' ws ( (has_glyph_predicate | category_predicate | has_anchor_predicate) | comp_predicate )
 comp_predicate = 'not'?:n ws  '(' ws <letter+>:metric ws ('>='|'<='|'='|'<'|'>'):comparator ws (<digit+>|bracketed_metric):value ws ')' -> {'predicate': metric, 'comparator': comparator, 'value': value, 'inverted': n}
 has_anchor_predicate = 'not'?:n ws 'hasanchor(' barename:anchor ')' -> {'predicate': 'hasanchor', 'value': anchor["barename"], 'inverted':n }
 has_glyph_predicate = 'not'?:n ws 'hasglyph(' regex:replace ws barename:withs ')' -> {'predicate': 'hasglyph', 'value': {'replace': replace["regex"], 'with': withs["barename"]}, 'inverted':n }
+predicate = ws 'and' ws ( has_glyph_predicate | category_predicate | has_anchor_predicate | comp_predicate )
 category_predicate = 'not'?:n ws 'category(' barename:cat ')' -> {'predicate': 'category', 'value': cat["barename"], 'inverted':n }
 bracketed_metric = <letter+>:metric '(' <(letter|digit|"."|"_")+>:glyph ')' -> {'metric': metric, 'glyph': glyph}
-conjunction = glyphselector:l ws ('&'|'|'|'-'):conjunction ws primary:r -> {'conjunction': {"&":"and","|":"or","-":"subtract"}[conjunction], 'left': l, 'right': r}
 
 primary_paren = '(' ws primary:p ws ')' -> p
 primary = primary_paren | conjunction | glyphselector
 
+conjunction = glyphselector:l ws ('&'|'|'|'-'):conjunction ws primary:r -> {'conjunction': {"&":"and","|":"or","-":"subtract"}[conjunction], 'left': l, 'right': r}
+
 DefineClass_Args = classname:c ws '=' ws definition:d -> (c,d)
 definition = primary:g predicate*:p -> (g,p)
 
@@ -134,7 +145,13 @@ class DefineClass:
         predicates = definition[1]
         for p in predicates:
             glyphs = list(filter(lambda x: self.meets_predicate(x, p, parser), glyphs))
-        parser.fontfeatures.namedClasses[classname["classname"]] = tuple(glyphs)
+        try:
+            parser.fontfeatures.namedClasses[classname["classname"]] = tuple(glyphs)
+        except ValueDuplicationError as e:
+            import warnings
+            warnings.warn("Could not define class %s as it contains the same"
+                    " glyphs as an existing class %s" % (classname["classname"],
+                        parser.fontfeatures.namedClasses.inverse[tuple(glyphs)]))
 
     @classmethod
     def resolve_definition(self, parser, primary):
