@@ -37,6 +37,8 @@ class GlyphSelector:
             returned = "/" + self.selector["regex"] + "/"
         elif "unicodeglyph" in self.selector:
             returned = "U+%04X" % self.selector["unicodeglyph"]
+        elif "unicoderange" in self.selector:
+            returned = "U+%04X=>U+%04X" % self.selector["unicoderange"].start, self.selector["unicoderange"].stop
         elif "inlineclass" in self.selector:
             items = [
                 GlyphSelector(i, (), self.location)
@@ -61,7 +63,7 @@ class GlyphSelector:
     def resolve(self, fontfeatures, font, mustExist=True):
         returned = []
         assert isinstance(font, Font)
-        glyphs = font.keys()
+        glyphs = font.exportedGlyphs()
         if "barename" in self.selector:
             returned = [self.selector["barename"]]
         elif "unicodeglyph" in self.selector:
@@ -73,6 +75,15 @@ class GlyphSelector:
                     % (cp, self.location)
                 )
             returned = [glyph]
+        elif "unicoderange" in self.selector:
+            returned = list(
+                collapse(
+                    [
+                        GlyphSelector({"unicodeglyph": i}, (), self.location).resolve(fontfeatures, font)
+                        for i in self.selector["unicoderange"]
+                    ]
+                )
+            )
         elif "inlineclass" in self.selector:
             returned = list(
                 collapse(
@@ -112,8 +123,8 @@ class GlyphSelector:
                     plural = "s"
                 glyphstring = ", ".join(notFound)
                 warnings.warn(
-                    "# Couldn't find glyph%s '%s' in font (at %s)"
-                    % (plural, glyphstring, self.location)
+                    "# Couldn't find glyph%s '%s' in font (%s at %s)"
+                    % (plural, glyphstring, self.as_text(), self.location)
                 )
         return list(returned)
 
@@ -136,16 +147,22 @@ comment = '#' rest_of_line ws?
 verb = <letter+>:x ?(x in self.valid_verbs) -> x
 
 # Ways of specifying glyphs
-classname = '@' <(letter|"_")+>:b  -> {"classname": b}
-barename = <(letter|digit|"."|"_")+ (("."|"_"|"-") (letter|digit)+)*>:b -> {"barename": b}
-hexdigit = anything:x ?(x in '0123456789abcdefABCDEF') -> x
-unicodeglyphname = 'U+' <hexdigit+>:u -> {"unicodeglyph": int(u,16) }
+classname = '@' barename:b -> {"classname": b["barename"]}
+
+startglyphname = anything:x ?(x in "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_")
+midglyphname = anything:x ?(x in "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_.-")
+# barename = <(letter|digit|"_"|'-')+>:b -> {"barename": b}
+barename = <startglyphname midglyphname*>:b -> {"barename": b}
 inlineclass_member = (barename|classname):m ws? -> m
 inlineclass_members = inlineclass_member+
 inlineclass = '[' ws inlineclass_members:m ']' -> {"inlineclass": m}
 regex = '/' <(~'/' anything)+>:r '/' -> {"regex": r}
+hexdigit = anything:x ?(x in '0123456789abcdefABCDEF') -> x
+unicodechar = 'U+' <hexdigit+>:u -> int(u,16)
+unicoderange = unicodechar:s '=>' unicodechar:e -> {"unicoderange": range(s,e+1)}
+unicodeglyphname = unicodechar:u -> {"unicodeglyph": u }
 glyphsuffix = ('.'|'~'):suffixtype <(letter|digit|"_")+>:suffix -> {"suffixtype":suffixtype, "suffix":suffix}
-glyphselector = (unicodeglyphname | regex | barename | classname | inlineclass ):g glyphsuffix*:s -> GlyphSelector(g,s, self.input.position)
+glyphselector = ( unicoderange | unicodeglyphname | regex | barename | classname | inlineclass ):g glyphsuffix*:s -> GlyphSelector(g,s, self.input.position)
 
 # Number things
 bareinteger = ('-'|'+')?:sign <digit+>:i -> (-int(i) if sign == "-" else int(i))
@@ -187,6 +204,8 @@ fee_value_record_member = ("xAdvance"| "xPlacement" | "yAdvance" | "yPlacement")
         self.font_modified = False
         self.variables = {}
         self._rebuild_parser()
+        # Set up GDEF table from font
+        self.fontfeatures.setGlyphClassesFromFont(self.font)
         for plugin in self.DEFAULT_PLUGINS:
             self._load_plugin(plugin)
 
